/*
 * Copyright (C) 2010-2013 jeanfi@gmail.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>


#include <glib.h>
#include <json/json.h>

#include "http.h"
#include "io.h"
#include "log.h"
#include "ttrss_wsasync.h"
#include "url.h"

static struct feed **data;
static char *cache_dir;

static const char *get_cache_dir()
{
	char *home;

	if (!cache_dir) {
		home = getenv("HOME");

		if (!home)
			return NULL;

		cache_dir = path_append(home, ".prss/cache");
		mkdirs(cache_dir, 0777);
	}

	return cache_dir;
}

static void file_set_content(const char *path, const char *content)
{
	FILE *fp;

	log_debug("file_set_content(): path=%s", path);

	fp = fopen(path, "w");
	if (fp) {
		fwrite(content, 1, strlen(content), fp);
		fclose(fp);
	}
}

static gchar *content_get_path(const struct headline *h)
{
	const char *cache_dir;

	cache_dir = get_cache_dir();
	if (cache_dir)
		return g_strdup_printf("%s/%d", cache_dir, h->id);

	return NULL;
}

static int is_content_cached(const struct headline *h)
{
	struct stat s;
	char *path;
	int result;

	path = content_get_path(h);

	if (stat(path, &s) == -1)
		result = 0;
	else
		result = 1;

	g_free(path);

	return result;
}

char *ttrss_get_headline_content(struct headline *h)
{
	char *path, *content;

	path = content_get_path(h);
	if (path)
		content = file_get_content(path);
	else
		content = NULL;

	if (!content) {
		content = ws_get_article_content(h->id);
		file_set_content(path, content);
	}

	g_free(path);

	return content;
}

struct feed **ttrss_get_feeds()
{
	data = ws_update_feeds(data);

	return data;
}

struct headline **ttrss_feed_get_headlines(struct feed *f)
{
	if (!f->headlines)
		ws_update_headlines(f);

	return f->headlines;
}

void ttrss_set_article_unread(int id, int unread)
{
	log_debug("ttrss_set_article_unread(%d,%d)", id, unread);

	ws_async_set_article_unread(id, unread);

	log_debug("ttrss_set_article_unread(%d,%d)", id, unread);
}

void ttrss_set_config(const char *url, const char *user, const char *pwd)
{
	feeds_free(data);
	data = NULL;
	ws_set_config(url, user, pwd);
}

struct feed *ttrss_get_feed(int id)
{
	return feeds_get_feed(data, id);
}

struct headline *ttrss_get_headline(int id)
{
	return feeds_get_headline(data, id);
}

void ttrs_download_headline_content(struct feed **feeds)
{
	struct feed **fcur;
	struct headline **hcur;

	for (fcur = feeds; *fcur; fcur++) {
		hcur = ttrss_feed_get_headlines(*fcur);

		while (hcur && *hcur) {
			if (!is_content_cached(*hcur))
				free(ttrss_get_headline_content(*hcur));
			hcur++;
		}
	}
}
